// Parametric ball joint in OpenSCAD by Peter Uithoven 
// Based on a design by Erik de Bruijn (https://www.youmagine.com/designs/parametric-ball-joint)
// Based on a design by Makerblock ( http://makerblock.com/2010/03/blender-help/ )

ballDiameter = 10; // diameter of the ball joint
ballR = ballDiameter/2; 
jointSpacing = 0; // some space between them?
jointThickness = 1.5; // thickness of the arms
jointNumArms = 4; // how many arms do you want?
jointArmWidth = 2; // actually: how much is removed from the arms Larger values will remove more
jointArmHeight = ballDiameter*0.5;
jointCutOffTop = ballDiameter*0.4; //*0.8;
footR = ballR*0.8; // diameter of foot
footHeight = footR*0.6;

//render settings
$fs=0.8; // def 1, 0.2 is high res
$fa=4;//def 12, 3 is very nice
o = 0.1; // slight overlap for boolean operations


print();
//demo(); // turn on animation, FPS 15, steps 200

module demo() {
  ball(ballDiameter,jointSpacing);
  rotate([180,0,0]) rotate([sin($t*720)*28,cos($t*360*4)*28,cos($t*360*2)*20]) joint(ballDiameter,jointSpacing,jointThickness,jointNumArms,jointArmWidth,jointArmHeight,jointCutOffTop);
}
module print() {
  translate([ballDiameter+10,0,0]) ballWithFoot(ballDiameter,footR,footHeight);
  jointWithFoot(ballDiameter,jointSpacing,jointThickness,jointNumArms,jointArmWidth,jointArmHeight,jointCutOffTop,footR);
}

module ball(ballDiameter,jointSpacing) {
  ballR = ballDiameter/2-jointSpacing/2; 
  sphere(r=ballR);
}
module ballWithFoot(ballDiameter,footR,footHeight) {
  ballR = ballDiameter/2; 
  translate([0,0,ballR+footHeight/2]) {
    ball(ballDiameter,jointSpacing);
    translate([0,0,-ballR]) cylinder(r1=footR,r2=footR*0.8,h=footHeight/2);
    translate([0,0,-ballR-footHeight/2]) cylinder(r=footR,h=footHeight/2);
  }
}

module jointBase(ballDiameter,jointSpacing,jointThickness) {
  sphere(r=ballR+jointSpacing/2+jointThickness);
}
module jointCutout(ballDiameter,jointSpacing,jointThickness,jointNumArms,jointArmWidth,jointArmHeight,jointCutOffTop) {
  ballR = ballDiameter/2; 
  outerDiameter = ballDiameter+jointSpacing+jointThickness*2;
  // cut out ball
  sphere(r=ballR+jointSpacing/2);
  // cut off top 
  translate([0,0,outerDiameter-jointCutOffTop]) cube([outerDiameter*2,outerDiameter*2,outerDiameter],center=true);
  // cut out arms
  intersection() {
    for(i=[0:jointNumArms]) rotate([0,0,360/jointNumArms*i]) {
      //translate([-jointArmWidth/2,0,-jointArmHeight/2]) {
      translate([-jointArmWidth/2,0,-jointArmHeight+outerDiameter/2-jointCutOffTop+o]) {
        cube([jointArmWidth,outerDiameter,jointArmHeight+o]);
      }
    }
    sphere(d=outerDiameter+o*2);
  }
}
// optional cutout to make joint flexible in solid shape
module jointCutoutOuter(ballDiameter,jointSpacing,jointThickness,thickness,depth) {
  //ballR = ballDiameter/2; 
  innerDiameter = ballDiameter+jointSpacing+jointThickness*2;
  outerDiameter = innerDiameter+jointThickness+thickness;
  
  difference() {
    sphere(d=outerDiameter);
    sphere(d=innerDiameter);
    translate([0,0,-outerDiameter/2+innerDiameter/2-depth]) cube(outerDiameter+o,center=true);
  }
}
module joint(ballDiameter,jointSpacing,jointThickness,jointNumArms,jointArmWidth,jointArmHeight,jointCutOffTop) {
  difference() {
    jointBase(ballDiameter,jointSpacing,jointThickness);
    jointCutout(ballDiameter,jointSpacing,jointThickness,jointNumArms,jointArmWidth,jointArmHeight,jointCutOffTop);
  }
}
module jointWithFoot(ballDiameter,jointSpacing,jointThickness,jointNumArms,jointArmWidth,jointArmHeight,jointCutOffTop,footR) {
  outerDiameter = ballDiameter+jointSpacing+jointThickness*2;
  translate([0,0,outerDiameter/2]) difference() {
    union() {
      jointBase(ballDiameter,jointSpacing,jointThickness);
      // foot
      translate([0,0,-outerDiameter/2]) cylinder(r=footR,h=outerDiameter/2);
    }
    jointCutout(ballDiameter,jointSpacing,jointThickness,jointNumArms,jointArmWidth,jointArmHeight,jointCutOffTop);
  }
}