// title: UM2 camera mount
// author: Peter Uithoven
// license: Creative Commons CC BY
// URL: 
// revision: 0.000
// tags: camera,um2,mount

caseSpacing = 0.5;
cameraWidth = 32+caseSpacing;
cameraHeight = 32+caseSpacing;
cameraDepth = 4;
thickness = 1.5;
caseWidth = cameraWidth+thickness*2;
caseHeight = cameraHeight+thickness*2;
caseDepth = cameraDepth+thickness*2;

roundRadius = 2;
boltR = 2.5/2;
boltDistance = 28; // distance between bolt's centers
JSTConW = 12;
JSTConH = 5.5+thickness;
JSTConD = caseDepth-thickness*2;
JSTConX = thickness+9.5; 
JSTConY = thickness*2+cameraHeight-JSTConH; //thickness+cameraHeight-JSTConH; //-5;
JSTConZ = thickness;

use<../ballJoint.scad>

ballDiameter = 10; // diameter of the ball joint
ballR = ballDiameter/2; 
jointSpacing = -0.2; // some space between them?
jointThickness = thickness; // thickness of the arms
jointNumArms = 4; // how many arms do you want?
jointArmWidth = 2; // actually: how much is removed from the arms Larger values will remove more
jointArmHeight = ballDiameter*0.5;
jointCutOffTop = ballDiameter*0.45;
jointDiameter = ballDiameter+jointThickness*2+jointSpacing;

ballX = caseWidth/2;
ballY = -ballR*0.8;
ballZ = caseDepth+ballR*0.2; //0.4;

o = 1; //0.1; // slight overlap
$fs=0.5; 
$fa=5; 

translate([0,0,caseHeight]) rotate([-90,0,0]) case(); 

//include<UM_camera_holder.scad>
//mirror([0,1,0]) translate([-thickness,-caseDepth,caseHeight-thickness]) rotate([-90,0,0]) case(); 
  
  
//cameraHolder();
//module cameraHolder() {
//  rotate([0,0,180]) {
//    include<UM_camera_holder.scad>
//  }
//}

module case() {
  difference() {
    union() {
      translate([thickness,thickness]) linear_extrude(caseDepth) {
        minkowski() {
          roundedSquare([cameraWidth,cameraHeight]);
          circle(d=thickness*2);
        }
      }
      translate([ballX,ballY,ballZ]) {
        ball(ballDiameter,jointSpacing);
        translate([0,ballR,ballY+1]) {
          scale([1.5,1,1.2]) ball(ballDiameter*0.75,jointSpacing);
        } 
      }
    }
    translate([thickness,thickness,thickness]) linear_extrude(caseDepth*3) {
      roundedSquare([cameraWidth,cameraHeight]);
    }
    translate([JSTConX,JSTConY,-o]) {
      cube([JSTConW,JSTConH+o,JSTConD+o]);
    }
    //translate([caseWidth/2,0,caseDepth+3]) sphere(d=10.5);
  }
}

module cameraBackPlate() {
//    // JST connector pins
//    translate([JSTConX,panelHeight-cameraHeight]) square([JSTConW,JSTConH]);
}

/////////////////////////////// UTILS ///////////////////////////////
module roundedSquare(pos=[10,10],rounding=2,center=false) {
  offset = (center)? 0 : rounding;
  translate([offset,offset]) minkowski() {
    square([pos[0]-rounding*2,pos[1]-rounding*2],center=center);
    circle(rounding);
  }
}
// clone children and put a disX (and disY) between them
module cloneMirrored(disX=0,disY=0) {
  translate([ disX/2, disY/2, 0]) children();
  translate([-disX/2, disY/2, 0]) scale([-1,1,1]) children();
  if(disY > 0) {
    translate([ disX/2, -disY/2, 0]) children();
    translate([-disX/2, -disY/2, 0]) scale([-1,1,1]) children();
  }
}
module ellipse(width, height) {
  scale([1, height/width, 1]) circle(r=width/2);
}