// title: UM camera holder
// author: Peter Uithoven
// license: Creative Commons CC BY
// URL: 
// revision: 0.000
// tags: camera,um2,mount

cameraWidth = 32;
cameraHeight = 32;
cameraZRotate = 45;
thickness = 1.5;
cameraHoleW = 17.5;
cameraHoleH = 16.5;
cameraHandleW = 4.5; // handle that mount camera mount width
cameraHandleH = 4; // handle that mount camera mount height
cameraHandleD = 4; // handle that mount camera mount depth

roundRadius = 2;
boltR = 2.5/2;
boltDistance = 28; // distance between bolt's centers
JSTConW = cameraWidth-10*2;
JSTConH = 4;

o = 0.1; // slight overlap
$fs=0.5; 
$fa=5; 

use<../ballJoint.scad>

ballDiameter = 10; // diameter of the ball joint
ballR = ballDiameter/2; 
jointSpacing = -0.2; // some space between them?
jointThickness = thickness; // thickness of the arms
jointNumArms = 4; // how many arms do you want?
jointArmWidth = 2; // actually: how much is removed from the arms Larger values will remove more
jointArmHeight = ballDiameter*0.5;
jointCutOffTop = ballDiameter*0.45;
jointDiameter = ballDiameter+jointThickness*2+jointSpacing;

rotate([-90,0,0]) translate([0,-cameraHeight,0]) cameraHolder();

////cool down pole
//translate([-10,0,0]) {
//  cylinder(r1=3,r2=1,h=ballR);
//  cylinder(r=1,h=cameraHeight+ballDiameter);
//}

module cameraHolder() {
  cameraHolderPlate();
  translate([cameraWidth/2,-ballR+1,-2]) {
    ball(ballDiameter,jointSpacing);
  }
  difference() {
    translate([cameraWidth/2,0,0]) scale([1.2,2,1]) {
      ball(ballDiameter*0.60,jointSpacing);
    }
    translate([-o,-ballDiameter,thickness]) cube([cameraWidth,cameraHeight+ballDiameter,20]);
  }
}
module cameraHolderPlate() {
  linear_extrude(thickness) difference() {
    roundedSquare([cameraWidth,cameraHeight]);
    // JST connector pins
    translate([(cameraWidth-JSTConW)/2,cameraHeight-JSTConH]) square([JSTConW,JSTConH+o]);
    // Camera
    translate([(cameraWidth-cameraHoleW)/2,(cameraHeight-cameraHoleH)/2]) square([cameraHoleW,cameraHoleH]);
    // Camera handles
    translate([cameraWidth/2,(cameraHeight-cameraHandleH)/2]) cloneMirrored(cameraHoleW-o*2) square([cameraHandleW+o,cameraHandleH]);
    // bolt holes
    translate([cameraWidth/2,cameraHeight/2]) cloneMirrored(boltDistance,boltDistance) circle(boltR);
  }
}

/////////////////////////////// UTILS ///////////////////////////////
module roundedSquare(pos=[10,10],rounding=2,center=false) {
	offset = (center)? 0 : rounding;
	translate([offset,offset]) minkowski() {
		square([pos[0]-rounding*2,pos[1]-rounding*2],center=center);
		circle(rounding);
	}
}
// clone children and put a disX (and disY) between them
module cloneMirrored(disX=0,disY=0) {
	translate([ disX/2, disY/2, 0]) children();
	translate([-disX/2, disY/2, 0]) scale([-1,1,1]) children();
	if(disY > 0) {
		translate([ disX/2, -disY/2, 0]) children();
		translate([-disX/2, -disY/2, 0]) scale([-1,1,1]) children();
	}
}
module ellipse(width, height) {
  scale([1, height/width, 1]) circle(r=width/2);
}