// title: UM camera mount hook
// author: Peter Uithoven
// license: Creative Commons CC BY
// URL: 
// revision: 0.000
// tags: camera,um2,mount

ballDiameter = 10; // diameter of the ball joint
ballR = ballDiameter/2; 
jointSpacing = -0.1; // some space between them?
jointThickness = 1.2; // thickness of the arms
jointNumArms = 4; // how many arms do you want?
jointArmWidth = 2; // actually: how much is removed from the arms Larger values will remove more
jointArmHeight = ballDiameter*0.5;
jointCutOffTop = ballDiameter*0.35;
jointDiameter = ballDiameter+jointThickness*2+jointSpacing*2;
jointOuterDepth = jointCutOffTop+jointArmHeight;
jointSpace = 1;

thickness = 1.5;
panelThickness = 6;
panelHeight = 35+1;
topHookDepth = 1; //4;
bottomHookDepth = 2; //1;
width = 25;

jointX = width/2;
jointY = thickness;
jointZ = thickness+panelHeight+ballR; //+armHeight+ballR-2;
jointAngle = 45;

boltTopD = 7; 
boltTopH = 2;
boltPos = 10; // center from top of panel

o = 0.1; // slight overlap
$fs=0.5; 
$fa=5; 

use<../ballJoint.scad>

handle();

////cool down pole
//translate([-10,0,0]) {
//  cylinder(r1=3,r2=1,h=ballR);
//  cylinder(r=1,h=jointZ+ballR);
//}

module handle() {
  difference() {
    hull() {
      cube([width,panelThickness+thickness*2,panelHeight+thickness*2]);
      translate([jointX,jointY,jointZ]) {
        jointBase(ballDiameter,jointSpacing,jointThickness); 
      }
    }
    translate([jointX,jointY,jointZ]) rotate([jointAngle,0,0]){
      jointCutout(ballDiameter,jointSpacing,jointThickness,jointNumArms,jointArmWidth,jointArmHeight,jointCutOffTop); 
      jointCutoutOuter(ballDiameter,jointSpacing,jointThickness,jointSpace,jointOuterDepth);
    }
    translate([-o,thickness,thickness]) cube([width+o*2,panelThickness,panelHeight]);
    translate([-o,thickness,thickness+topHookDepth]) cube([width+o*2,panelThickness+thickness+o,panelHeight-topHookDepth-bottomHookDepth]);
    
    // bolt cut out
    translate([width/4,thickness,boltPos]) rotate([90,0,0]) hull() cloneMirrored(width/2) {
      translate([0,0,-o])cylinder(d=boltTopD,h=boltTopH/2+o);
      translate([0,0,boltTopH/2]) resize([boltTopD,boltTopD,boltTopH]) sphere(d=boltTopD);
    }
  }
  
  // um front top panel bar
  %translate([-width*0.5,thickness,thickness]) cube([width*2,panelThickness,panelHeight]);
}

/////////////////////////////// UTILS ///////////////////////////////
module roundedSquare(pos=[10,10],rounding=2,center=false) {
	offset = (center)? 0 : rounding;
	translate([offset,offset]) minkowski() {
		square([pos[0]-rounding*2,pos[1]-rounding*2],center=center);
		circle(rounding);
	}
}
// clone children and put a disX (and disY) between them
module cloneMirrored(disX=0,disY=0) {
	translate([ disX/2, disY/2, 0]) children();
	translate([-disX/2, disY/2, 0]) scale([-1,1,1]) children();
	if(disY > 0) {
		translate([ disX/2, -disY/2, 0]) children();
		translate([-disX/2, -disY/2, 0]) scale([-1,1,1]) children();
	}
}
module ellipse(width, height) {
  scale([1, height/width, 1]) circle(r=width/2);
}