// title: UM2 camera mount
// author: Peter Uithoven
// license: Creative Commons CC BY
// URL: 
// revision: 0.000
// tags: camera,um2,mount

cameraWidth = 32;
cameraHeight = 32;
cameraZ = 100-cameraHeight/2; // to bottom of plate
cameraZRotate = 45;
cameraDownwardsAngle = 30; // downwards
thickness = 1.5;
cameraHoleW = 17.5;
cameraHoleH = 16;
cameraHandleW = 4.5; // handle that mount camera mount width
cameraHandleH = 4; // handle that mount camera mount height
cameraHandleD = 4; // handle that mount camera mount depth
o = 0.1; // slight overlap
$fs=0.5; 
$fa=5; 

jointX = 31.5; // from inside of side panel
jointY = 14; // from hook, inside printer 
jointZ = 103.2; // from bottom of top hook

use<ballJoint.scad>

ballDiameter = 10; // diameter of the ball joint
ballR = ballDiameter/2; 
jointSpacing = 0; // some space between them?
jointThickness = thickness; // thickness of the arms
jointNumArms = 4; // how many arms do you want?
jointArmWidth = 2; // actually: how much is removed from the arms Larger values will remove more
jointArmHeight = ballDiameter*0.5;
jointCutOffTop = ballDiameter*0.45;
jointDiameter = ballDiameter+jointThickness*2+jointSpacing*2;
//cameraHolderPlate();
handle();
//cameraHolder();

//#translate([0,0,thickness]) cube([jointX,jointY,jointZ]);

module handle() {
  hookDepth = 6;
  hookWidth = 25;
  handleWidth = 25;
  handleHeight = jointZ+thickness+ballR;
  handleThickness = thickness;
  bottomWidth = 33;
  bottomHeight = 33;
  bottomHookThickness = handleThickness;
  
  rotate([90,0,0]) {
    // hook around top
    cube([hookWidth,hookDepth+handleThickness,handleThickness]);
    // middle
    cube([handleWidth,handleThickness,handleHeight]);
    // bottom 
    translate([0,0,handleHeight-bottomHeight]) {
      cube([bottomWidth,handleThickness,bottomHeight]);
      // bottom around side
      translate([bottomWidth,0,0]) cube([bottomHookThickness,hookDepth+handleThickness+bottomHookThickness,bottomHeight]);
      // bottom hook
      translate([bottomWidth-hookDepth,hookDepth+bottomHookThickness,0]) cube([hookDepth+bottomHookThickness,bottomHookThickness,bottomHeight]);
    }
    // joint
    translate([jointX,jointY,jointZ+handleThickness]) rotate([-90,0,45]) {
      joint(ballDiameter,jointSpacing,jointThickness,jointNumArms,jointArmWidth,jointArmHeight,jointCutOffTop);
    }
    // cool down tower
    translate([bottomWidth,0,0]) rotate([-90,0,0]) cylinder(r=1,h=hookDepth+handleThickness*2+ballDiameter);
  }
}

module cameraHolder() {
  //translate([0,0,cameraZ]) 
  //rotate([cameraDownwardsAngle,0,-90+cameraZRotate]) translate([-cameraWidth,0,-cameraHeight]) rotate([90,0,0]) cameraHolderPlate();
  //!rotate([90-cameraYRotate,0,0]) !cameraHolderPlate();
  ballY = cameraHeight*0.275;
  difference() {
    union() {
      cameraHolderPlate();
      // ball holder
//      translate([0,cameraHeight/2,0]) rotate([0,90,0]) {
//        cylinder(d=cameraHeight/2,h=(cameraWidth-cameraHoleW)/2);
//      }
      //translate([(cameraWidth-cameraHoleW)/4,ballY,0]) {
      
      //translate([cameraWidth+ballR-1,cameraHeight*0.25,0]) {
      translate([cameraWidth,cameraHeight*0.7,0]) {
        ball(ballDiameter*0.6);
        translate([ballR-1,0,-2]) ball(ballDiameter);
        //scale([1,1.4,1.60]) sphere(r=footR*1.5);
      }
    }
//    translate([(cameraWidth-cameraHoleW-cameraHandleW*2)/2,cameraHeight/2-cameraHandleH/2,-cameraHandleD+thickness]) {
//      cube([cameraHandleW*2+o,cameraHandleH,cameraHandleD+o]);
//    }
    translate([-o,0,thickness]) cube([cameraWidth+o+ballDiameter,cameraHeight,20]);
  }
}

module cameraHolderPlate() {
  // hold it from the back? 
  // slide in from side? but what about plugs? 
  // ballsocket! 
  
  w = cameraWidth; //+1?
  h = cameraHeight; //+1?
  d = 1.5; //+/-?
  roundRadius = 2;
  boltR = 2.5/2;
  boltDistance = 28; // distance between bolt's centers
  JSTConW = w-10*2;
  JSTConH = 4;
  
  linear_extrude(d) difference() {
    square([w,h]);
    // JST connector pins
    translate([(w-JSTConW)/2,h-JSTConH]) square([JSTConW,JSTConH+o]);
    // Camera
    translate([(w-cameraHoleW)/2,(h-cameraHoleH)/2]) square([cameraHoleW,cameraHoleH]);
    // Camera handles
    translate([w/2,(h-cameraHandleH)/2]) cloneMirrored(cameraHoleW-o*2) square([cameraHandleW+o,cameraHandleH]);
    // bolt holes
    translate([w/2,h/2]) cloneMirrored(boltDistance,boltDistance) circle(boltR);
  }
}

/////////////////////////////// UTILS ///////////////////////////////
module roundedSquare(pos=[10,10],rounding=2,center=false) {
	offset = (center)? 0 : rounding;
	translate([offset,offset]) minkowski() {
		square([pos[0]-rounding*2,pos[1]-rounding*2],center=center);
		circle(rounding);
	}
}
// clone children and put a disX (and disY) between them
module cloneMirrored(disX=0,disY=0) {
	translate([ disX/2, disY/2, 0]) children();
	translate([-disX/2, disY/2, 0]) scale([-1,1,1]) children();
	if(disY > 0) {
		translate([ disX/2, -disY/2, 0]) children();
		translate([-disX/2, -disY/2, 0]) scale([-1,1,1]) children();
	}
}